@@ -80,43 +80,6 @@ impl ResultRSEImm12 {
     }
 }
 
-/// A lowering result: register, register-shift, or logical immediate form.
-/// An SSA value can always be lowered into one of these options; the register form is the
-/// fallback.
-#[derive(Clone, Debug)]
-pub(crate) enum ResultRSImmLogic {
-    Reg(Reg),
-    RegShift(Reg, ShiftOpAndAmt),
-    ImmLogic(ImmLogic),
-}
-
-impl ResultRSImmLogic {
-    fn from_rs(rse: ResultRS) -> ResultRSImmLogic {
-        match rse {
-            ResultRS::Reg(r) => ResultRSImmLogic::Reg(r),
-            ResultRS::RegShift(r, s) => ResultRSImmLogic::RegShift(r, s),
-        }
-    }
-}
-
-/// A lowering result: register or immediate shift amount (arg to a shift op).
-/// An SSA value can always be lowered into one of these options; the register form is the
-/// fallback.
-#[derive(Clone, Debug)]
-pub(crate) enum ResultRegImmShift {
-    Reg(Reg),
-    ImmShift(ImmShift),
-}
-
-impl ResultRegImmShift {
-    pub fn unwrap_reg(self) -> Reg {
-        match self {
-            ResultRegImmShift::Reg(r) => r,
-            _ => panic!("Unwrapped ResultRegImmShift, expected reg, got: {:?}", self),
-        }
-    }
-}
-
 //============================================================================
 // Lowering: convert instruction inputs to forms that we can use.
 
@@ -488,37 +451,6 @@ pub(crate) fn put_input_in_rse_imm12<C: LowerCtx<I = Inst>>(
     ResultRSEImm12::from_rse(put_input_in_rse(ctx, input, narrow_mode))
 }
 
-pub(crate) fn put_input_in_rs_immlogic<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    input: InsnInput,
-    narrow_mode: NarrowValueMode,
-) -> ResultRSImmLogic {
-    if let Some(imm_value) = input_to_const(ctx, input) {
-        let ty = ctx.input_ty(input.insn, input.input);
-        let ty = if ty_bits(ty) < 32 { I32 } else { ty };
-        if let Some(i) = ImmLogic::maybe_from_u64(imm_value, ty) {
-            return ResultRSImmLogic::ImmLogic(i);
-        }
-    }
-
-    ResultRSImmLogic::from_rs(put_input_in_rs(ctx, input, narrow_mode))
-}
-
-pub(crate) fn put_input_in_reg_immshift<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    input: InsnInput,
-    shift_width_bits: usize,
-) -> ResultRegImmShift {
-    if let Some(imm_value) = input_to_const(ctx, input) {
-        let imm_value = imm_value & ((shift_width_bits - 1) as u64);
-        if let Some(immshift) = ImmShift::maybe_from_u64(imm_value) {
-            return ResultRegImmShift::ImmShift(immshift);
-        }
-    }
-
-    ResultRegImmShift::Reg(put_input_in_reg(ctx, input, NarrowValueMode::None))
-}
-
 //============================================================================
 // ALU instruction constructors.
 
@@ -553,57 +485,6 @@ pub(crate) fn alu_inst_imm12(op: ALUOp, rd: Writable<Reg>, rn: Reg, rm: ResultRS
     }
 }
 
-pub(crate) fn alu_inst_immlogic(
-    op: ALUOp,
-    rd: Writable<Reg>,
-    rn: Reg,
-    rm: ResultRSImmLogic,
-) -> Inst {
-    match rm {
-        ResultRSImmLogic::ImmLogic(imml) => Inst::AluRRImmLogic {
-            alu_op: op,
-            rd,
-            rn,
-            imml,
-        },
-        ResultRSImmLogic::Reg(rm) => Inst::AluRRR {
-            alu_op: op,
-            rd,
-            rn,
-            rm,
-        },
-        ResultRSImmLogic::RegShift(rm, shiftop) => Inst::AluRRRShift {
-            alu_op: op,
-            rd,
-            rn,
-            rm,
-            shiftop,
-        },
-    }
-}
-
-pub(crate) fn alu_inst_immshift(
-    op: ALUOp,
-    rd: Writable<Reg>,
-    rn: Reg,
-    rm: ResultRegImmShift,
-) -> Inst {
-    match rm {
-        ResultRegImmShift::ImmShift(immshift) => Inst::AluRRImmShift {
-            alu_op: op,
-            rd,
-            rn,
-            immshift,
-        },
-        ResultRegImmShift::Reg(rm) => Inst::AluRRR {
-            alu_op: op,
-            rd,
-            rn,
-            rm,
-        },
-    }
-}
-
 //============================================================================
 // Lowering: addressing mode support. Takes instruction directly, rather
 // than an `InsnInput`, to do more introspection.
@@ -1580,43 +1461,6 @@ pub(crate) fn materialize_bool_result<C: LowerCtx<I = Inst>>(
     }
 }
 
-pub(crate) fn lower_shift_amt<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    amt_input: InsnInput,
-    dst_ty: Type,
-    tmp_reg: Writable<Reg>,
-) -> ResultRegImmShift {
-    let amt_ty = ctx.input_ty(amt_input.insn, amt_input.input);
-
-    match (dst_ty, amt_ty) {
-        // When shifting for amounts larger than the size of the type, the CLIF shift
-        // instructions implement a "wrapping" behaviour, such that an i8 << 8 is
-        // equivalent to i8 << 0
-        //
-        // On i32 and i64 types this matches what the aarch64 spec does, but on smaller
-        // types (i16, i8) we need to do this manually, so we wrap the shift amount
-        // with an AND instruction
-        (I16 | I8, _) => {
-            // We can ignore the top half of the shift amount register if the type is I128
-            let amt_reg = put_input_in_regs(ctx, amt_input).regs()[0];
-            let mask = (ty_bits(dst_ty) - 1) as u64;
-            ctx.emit(Inst::AluRRImmLogic {
-                alu_op: ALUOp::And32,
-                rd: tmp_reg,
-                rn: amt_reg,
-                imml: ImmLogic::maybe_from_u64(mask, I32).unwrap(),
-            });
-            ResultRegImmShift::Reg(tmp_reg.to_reg())
-        }
-        // TODO: We can use immlogic for i128 types here
-        (I128, _) | (_, I128) => {
-            // For I128 shifts, we need a register without immlogic
-            ResultRegImmShift::Reg(put_input_in_regs(ctx, amt_input).regs()[0])
-        }
-        _ => put_input_in_reg_immshift(ctx, amt_input, ty_bits(dst_ty)),
-    }
-}
-
 /// This is target-word-size dependent.  And it excludes booleans and reftypes.
 pub(crate) fn is_valid_atomic_transaction_ty(ty: Type) -> bool {
     match ty {
@@ -1629,7 +1473,7 @@ pub(crate) fn emit_atomic_load<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
     rt: Writable<Reg>,
     insn: IRInst,
-) {
+) -> Inst {
     assert!(ctx.data(insn).opcode() == Opcode::AtomicLoad);
     let inputs = insn_inputs(ctx, insn);
     let rn = put_input_in_reg(ctx, inputs[0], NarrowValueMode::None);
@@ -1638,7 +1482,7 @@ pub(crate) fn emit_atomic_load<C: LowerCtx<I = Inst>>(
     // We're ignoring the result type of the load because the LoadAcquire will
     // explicitly zero extend to the nearest word, and also zero the high half
     // of an X register.
-    ctx.emit(Inst::LoadAcquire { access_ty, rt, rn });
+    Inst::LoadAcquire { access_ty, rt, rn }
 }
 
 fn load_op_to_ty(op: Opcode) -> Option<Type> {
@@ -1689,211 +1533,6 @@ pub(crate) fn lower_load<
     f(ctx, rd, elem_ty, mem)
 }
 
-pub(crate) fn emit_shl_i128<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    src: ValueRegs<Reg>,
-    dst: ValueRegs<Writable<Reg>>,
-    amt: Reg,
-) {
-    let src_lo = src.regs()[0];
-    let src_hi = src.regs()[1];
-    let dst_lo = dst.regs()[0];
-    let dst_hi = dst.regs()[1];
-
-    //     mvn     inv_amt, amt
-    //     lsr     tmp1, src_lo, #1
-    //     lsl     tmp2, src_hi, amt
-    //     lsr     tmp1, tmp1, inv_amt
-    //     lsl     tmp3, src_lo, amt
-    //     tst     amt, #0x40
-    //     orr     tmp2, tmp2, tmp1
-    //     csel    dst_hi, tmp3, tmp2, ne
-    //     csel    dst_lo, xzr, tmp3, ne
-
-    let xzr = writable_zero_reg();
-    let inv_amt = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp1 = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp2 = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp3 = ctx.alloc_tmp(I64).only_reg().unwrap();
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::OrrNot32,
-        rd: inv_amt,
-        rn: xzr.to_reg(),
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRImmShift {
-        alu_op: ALUOp::Lsr64,
-        rd: tmp1,
-        rn: src_lo,
-        immshift: ImmShift::maybe_from_u64(1).unwrap(),
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Lsl64,
-        rd: tmp2,
-        rn: src_hi,
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Lsr64,
-        rd: tmp1,
-        rn: tmp1.to_reg(),
-        rm: inv_amt.to_reg(),
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Lsl64,
-        rd: tmp3,
-        rn: src_lo,
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRImmLogic {
-        alu_op: ALUOp::AndS64,
-        rd: xzr,
-        rn: amt,
-        imml: ImmLogic::maybe_from_u64(64, I64).unwrap(),
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Orr64,
-        rd: tmp2,
-        rn: tmp2.to_reg(),
-        rm: tmp1.to_reg(),
-    });
-
-    ctx.emit(Inst::CSel {
-        cond: Cond::Ne,
-        rd: dst_hi,
-        rn: tmp3.to_reg(),
-        rm: tmp2.to_reg(),
-    });
-
-    ctx.emit(Inst::CSel {
-        cond: Cond::Ne,
-        rd: dst_lo,
-        rn: xzr.to_reg(),
-        rm: tmp3.to_reg(),
-    });
-}
-
-pub(crate) fn emit_shr_i128<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    src: ValueRegs<Reg>,
-    dst: ValueRegs<Writable<Reg>>,
-    amt: Reg,
-    is_signed: bool,
-) {
-    let src_lo = src.regs()[0];
-    let src_hi = src.regs()[1];
-    let dst_lo = dst.regs()[0];
-    let dst_hi = dst.regs()[1];
-
-    //     mvn       inv_amt, amt
-    //     lsl       tmp1, src_lo, #1
-    //     lsr       tmp2, src_hi, amt
-    //     lsl       tmp1, tmp1, inv_amt
-    //     lsr/asr   tmp3, src_lo, amt
-    //     tst       amt, #0x40
-    //     orr       tmp2, tmp2, tmp1
-    //
-    //     if signed:
-    //         asr     tmp4, src_hi, #63
-    //         csel    dst_hi, tmp4, tmp3, ne
-    //     else:
-    //         csel    dst_hi, xzr, tmp3, ne
-    //
-    //     csel      dst_lo, tmp3, tmp2, ne
-
-    let xzr = writable_zero_reg();
-    let inv_amt = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp1 = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp2 = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp3 = ctx.alloc_tmp(I64).only_reg().unwrap();
-    let tmp4 = ctx.alloc_tmp(I64).only_reg().unwrap();
-
-    let shift_op = if is_signed {
-        ALUOp::Asr64
-    } else {
-        ALUOp::Lsr64
-    };
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::OrrNot32,
-        rd: inv_amt,
-        rn: xzr.to_reg(),
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRImmShift {
-        alu_op: ALUOp::Lsl64,
-        rd: tmp1,
-        rn: src_hi,
-        immshift: ImmShift::maybe_from_u64(1).unwrap(),
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Lsr64,
-        rd: tmp2,
-        rn: src_lo,
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Lsl64,
-        rd: tmp1,
-        rn: tmp1.to_reg(),
-        rm: inv_amt.to_reg(),
-    });
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: shift_op,
-        rd: tmp3,
-        rn: src_hi,
-        rm: amt,
-    });
-
-    ctx.emit(Inst::AluRRImmLogic {
-        alu_op: ALUOp::AndS64,
-        rd: xzr,
-        rn: amt,
-        imml: ImmLogic::maybe_from_u64(64, I64).unwrap(),
-    });
-
-    if is_signed {
-        ctx.emit(Inst::AluRRImmShift {
-            alu_op: ALUOp::Asr64,
-            rd: tmp4,
-            rn: src_hi,
-            immshift: ImmShift::maybe_from_u64(63).unwrap(),
-        });
-    }
-
-    ctx.emit(Inst::AluRRR {
-        alu_op: ALUOp::Orr64,
-        rd: tmp2,
-        rn: tmp2.to_reg(),
-        rm: tmp1.to_reg(),
-    });
-
-    ctx.emit(Inst::CSel {
-        cond: Cond::Ne,
-        rd: dst_hi,
-        rn: if is_signed { tmp4 } else { xzr }.to_reg(),
-        rm: tmp3.to_reg(),
-    });
-
-    ctx.emit(Inst::CSel {
-        cond: Cond::Ne,
-        rd: dst_lo,
-        rn: tmp3.to_reg(),
-        rm: tmp2.to_reg(),
-    });
-}
-
 pub(crate) fn emit_clz_i128<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
     src: ValueRegs<Reg>,
